#!/bin/sh
#
# This script is made to convert the file:
# http://standards.ieee.org/regauth/oui/oui.txt
# into a format that is used by the "setmac.sh"
# script.
#
# This script has been coded by Helith / www.helith.net
# Author: Rembrandt
#
# Check for the first two arguments
# input: Input file name (Default: oui.txt)
# output: Output file name (Default: list.txt)
#

input=$1
output=$2

#
# Display the usage dialogue
#

function usage {
	echo "Usage: convert_oui.sh [input] [output]"
	echo "Example: convert_oui.sh oui.txt list.txt\n"
	echo "You will need http://standards.ieee.org/regauth/oui/oui.txt"
	exit 0
}

#
# Make sure an input and ouput argument has been specified
#

if [ "$input" = ""  ] || [ "$output" = "" ]
then
	echo ">> Please specify input and ouput! <<"
	usage
	exit 0
fi

#
# Make sure the input file exists
#

if [ ! -e "$input" ]
then
	echo ">> Make sure the input file exists! <<"
	usage
	exit 0
fi

#
# Convert the file
#

a=1
while read -r line
do
	vendor=`echo $line | awk '{print $2}'`
	if [ "$vendor" = "(hex)" ]
	then
		part1=`echo $line | awk '{print $1}' | sed 's/-/ /g'`
		part2=`echo $line | awk '{print $3,$4,$5,$6}' | sed 's/,//g' | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
		list_line="$a = $part1 = $part2"
		a=`expr $a + 1`
		echo "$list_line" >> $output
	fi
done < $input

sort -k 7 $output > sorted.txt
mv sorted.txt $output
