#!/bin/sh
#
# Search a file list using the setmac format for vendors
# or specific MAC addresses!
#
# Default file to search is: list.txt, but it is possible
# to specify the file to search by appending "-f file"
# Example: sh search.sh -v cisco -f example.txt
#
# This script has been coded by Helith / www.helith.net
# Author: Rembrandt
#
# Variables are handled the following way.
#
# If the first argument is "-v" (vendor search), then
# arg2=<vendorname>
# arg3 can be "-f", if so arg4=<filename>
#
# If the first argument is "-m" (mac query), then
# arg2-4=<two byte blocks of the MAC address>
# arg5 can be "-f", if so arg6=<filename>
#

arg1=$1
arg2=$2
arg3=$3
arg4=$4
arg5=$5
arg6=$6

#
# Search the specified file for a vendor's MAC addresses
#

function vendorsearch {
	arg2=`echo "$arg2" | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`

	if [ "$arg3" = "-f" ] && [ -e "$arg4" ]
	then
		input=$arg4
	else
		input=list.txt
	fi

     	while read -r line
     	do
        	vendor=`echo "$line" | awk '{print $7}' | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
		if [ "$arg2" = "$vendor" ]
	then
		echo "$line"
	fi
	done < $input
	exit 0
}

#
# Search the specified file for one MAC address
#

function macsearch {
	arg2=`echo "$arg2" | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
	arg3=`echo "$arg3" | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
	arg4=`echo "$arg4" | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`

	if [ "$arg5" = "-f" ] && [ -e "$arg6" ]
	then
		input=$arg6
	else
		input=list.txt
	fi

	while read -r line
	do
		mac1=`echo "$line" | awk '{print $3}'`
		mac2=`echo "$line" | awk '{print $4}'`
		mac3=`echo "$line" | awk '{print $5}'`
		if [ "$arg2" = "$mac1" ] && [ "$arg3" = "$mac2" ] && [ "$arg4" = "$mac3" ]
		then
			echo $line
			exit 0
		fi
	done < $input
}

#
# Display the usage dialogue
#

function usage {
	echo "Usage: search.sh -v|m|h [search] -f [file]"
	echo "	-v	Search all MAC addresses related"
	echo "		to the specified vendor"
	echo "	-m	Search the vendor associated with"
	echo "		a specific MAC address"
	echo "	-h	Help (this one)"
	echo "	-f	Optional: You can specify a file in"
	echo "		setmac format to search in, default"
	echo "		is ' list.txt '"
	echo "Example: search.sh -v cisco -f example.txt"
	exit 0
}

#
# MAIN CODE
#

case "$arg1" in
	-m)	if [ "$arg2" = "" ]
		then
			echo ">> Please specify a valid MAC address! <<"
			usage
		fi
		macsearch
		;;
	-v)	if [ "$arg2" = "" ]
		then
			echo ">> Please specify a valid vendor name! <<"
			usage
		fi
		vendorsearch
		;;
	-h)	usage
		;;
	*)	echo ">> Invalid parameter or no parameter provided! <<"
		usage
		;;
esac
