#!/bin/sh
#
# Calculate MAC addresses randomly or based on vendor codes!
# Should work on: Darwin/DragonFly/FreeBSD/Linux/OpenBSD
#
# If your *NIX isn`t supported or need some modifications please let us know
# so that we`re able to make this script as portable as possible because for
# such easy tasks a C-Programm shouldn`t be needed at all. ;]
# But be patient because KSH is not that fast either.
# 
# Vendor codes (OUI): http://standards.ieee.org/regauth/oui/oui.txt
#
# This script has been coded by Helith / www.helith.net
# Author: Rembrandt
#
# Check for the first three arguments
# Option1: Parameter
# Option2: Vendor
# Option3: set
#

option=$1
option2=$2
option3=$3

#
# These arguments are optional and
# only important if you specify a file to use
# (Default: list.txt)
#
arg4=$4
arg5=$5

OS=`uname -s`

md5="md5"
random="/dev/arandom"

if [ "$OS" = "Linux" ]; then
	md5="md5sum"
	random="/dev/urandom"
fi

#
# LOCAL FUNCTIONS
#

#
# Display the usage dialogue
#
function usage {
	echo "Usage: setmac.sh -r|v|h [vendor] [set] -f [file]"
	echo "	-r	Random calculation of MAC address"
	echo "	-v	Vendor specific calculation of MAC address"
	echo "		Requires: Vendor number"
	echo "		To set the MAC address type 'set' after the number"
	echo "	-list	Print out the list of vendors and the number to use"
	echo "	-h	Help (this one)"
	echo "	-f	Optional: You can specify a file in setmac format"
	echo "		to use for generating vendors MAC addresses"
	echo "		(Default: list.txt)"
	echo "This would set a vendor specific MAC address by using the"
	echo "file ' example.txt ':"
	echo "Example: setmac.sh -v 1 -f example.txt"
	exit 0
}

#
# Generate two bytes randomly to be used for the MAC address
#
function mac {
	while [ $a != 7 ]
	do
		export match$a=`head -1 $random | $md5 | cut -b 1,2`
		a=`expr $a + 1`
	done
}

#
# Generate a random MAC address
#
function random {
	a=1
	mac
	MAC=""$match1":"$match2":"$match3":"$match4":"$match5":"$match6""
	echo "MAC address: $MAC"
	exit 0
}

#
# Generate a vendor specific MAC address
#
function vendor {
if [ "$option3" = "-f" ]
then
	input=$arg4
elif [ "$arg4" = "-f" ]
then
	input=$arg5
else
	input=list.txt
fi

while read -r line
do
	vendor=`echo $line | awk '{print $1}'`
	if [ "$option2" = "$vendor" ]
	then
		match1=`echo $line | awk '{print $3}'`
		match2=`echo $line | awk '{print $4}'`
		match3=`echo $line | awk '{print $5}'`
		a=4
		mac
		MAC=""$match1":"$match2":"$match3":"$match4":"$match5":"$match6""
		vendor=`echo $line | awk '{print $7,$8,$9,$10}'`
		echo "Vendor: $vendor"
		echo "MAC address: $MAC"
                exit 0
	fi
done < $input

	#
	# Enable the MAC address for the network interface
	#
	if [ "$option3" =  "set" ]
	then

		if [ "$OS" = "Darwin" ] || [ "$OS" = "OpenBSD" ]; then 
			CMD=lladdr
			NIC=`route show | grep default | awk '{print $7}'`
		elif [ "$OS" = "DragonFly" ] || [ "$OS" = "FreeBSD" ]; then
			CMD=link
			NIC=`route show | grep default | awk '{print $7}'`
		elif [ "$OS" = "Linux" ]; then
			CMD=ether
			NIC=`route | grep default | awk '{print $8}'`
		fi

		echo "Detected OS: $OS"
		echo "--------------------------------------------"
		echo "Setting the MAC address for your computer..."
		echo "--------------------------------------------"
		
		ifconfig $NIC down
		ifconfig $NIC $CMD $MAC up
	fi
	exit 0
}

#
# Display the list file and the number the user has to type
#
function list {
if [ "$option2" = "-f" ]
then
	input=$option3
else
	input=list.txt
fi

	echo "List of vendor codes:"
	while read -r line
	do
		echo -ne "$line"
		echo ""
	done < $input
	exit 0
}

#
# MAIN CODE
#

case "$option" in
	-r)	random
		;;
	-v)	if [ "$option2" = "" ]
		then
			echo ">> Please specify the vendor number! <<"
			usage
		fi
		vendor
		;;
	-h)	usage
		;;
	-list) list
		;;
	*)	echo ">> Invalid parameter or no parameter provided! <<"
		usage
		;;
esac
